#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import create_context_mask, create_full_mask
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_context_samples_clevr(args, scene):
    objects = [obj for obj in scene["objects"] if "mask" in obj]
    context_mask = create_context_mask(objects)

    sample = {"img_name": scene["image_filename"], "target": context_mask}
    return [sample]


def get_context_samples_mnist_dialog(args, scene):
    context_mask = create_full_mask(args)
    sample = {"img_name": scene["img_name"], "target": context_mask}
    return [sample]


def get_context_samples(args, scene):
    if args.experiment == Experiment.CLEVR:
        return get_context_samples_clevr(args, scene)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_context_samples_mnist_dialog(args, scene)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_context_dataset(args):
    samples = []
    for scene in tqdm(args.dataset["scenes"]):
        new_samples = get_context_samples(args, scene)
        samples.extend(new_samples)
    samples = get_max_sample(args, samples)
    data = {"samples": samples}
    write(args, data, module_name="context")
