#!/usr/bin/env python

import itertools
import random

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import create_context_mask, create_empty_mask
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_count_samples_clevr(args, scene):
    samples = []
    objects = [obj for obj in scene["objects"] if "mask" in obj]

    for i in range(1, len(objects) + 1):
        for subset in itertools.combinations(objects, i):
            subset = list(subset)
            mask = create_context_mask(subset)
            sample = {"img_name": scene["image_filename"], "mask": mask, "target": i}
            samples.append(sample)
    # manually add the empty mask to train count=0
    zero_sample = {
        "img_name": scene["image_filename"],
        "mask": create_empty_mask(args),
        "target": 0,
    }
    samples.append(zero_sample)
    return samples


def get_count_samples_mnist(args, scene):
    samples = []
    digits = scene["digits"]
    for i in range(1, len(digits) + 1):
        random.shuffle(digits)
        for subset in itertools.combinations(digits, i):
            subset = list(subset)
            mask = create_context_mask(subset)
            sample = {"img_name": scene["img_name"], "mask": mask, "target": i}
            samples.append(sample)
            break  # count can produce many samples so we only produce a single sample for each combination
    # manually add the empty mask to train count=0
    zero_sample = {
        "img_name": scene["img_name"],
        "mask": create_empty_mask(args),
        "target": 0,
    }
    samples.append(zero_sample)
    return samples


def get_count_samples(args, scene):
    if args.experiment == Experiment.CLEVR:
        return get_count_samples_clevr(args, scene)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_count_samples_mnist(args, scene)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_count_dataset(args):
    samples = []
    for scene in tqdm(args.dataset["scenes"]):
        new_samples = get_count_samples(args, scene)
        samples.extend(new_samples)
    samples = get_max_sample(args, samples)
    data = {"samples": samples}
    write(args, data, module_name="count")
