#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import combine_masks_for_find
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_find_category_samples_clevr(args, scene, category, concepts):
    samples = []
    objects = scene["objects"]
    for concept in concepts:
        input_mask, target_mask = combine_masks_for_find(args, objects, category, concept)
        sample = {
            "img_name": scene["image_filename"],
            "mask": input_mask,
            "concept": concept,
            "target": target_mask,
        }
        samples.append(sample)
    return samples


def get_find_category_samples_mnist(args, scene, category, concepts):
    samples = []
    digits = scene["digits"]
    for concept in concepts:
        input_mask, target_mask = combine_masks_for_find(args, digits, category, concept)
        sample = {
            "img_name": scene["img_name"],
            "mask": input_mask,  # is never used
            "concept": concept,
            "target": target_mask,
        }
        samples.append(sample)
    return samples


def get_find_category_samples(args, scene, category, concepts):
    if args.experiment == Experiment.CLEVR:
        return get_find_category_samples_clevr(args, scene, category, concepts)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_find_category_samples_mnist(args, scene, category, concepts)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_find_category_dataset(args):
    categories = args.config.categories
    for category in categories.keys():
        concepts = categories[category]
        samples = []
        for scene in tqdm(args.dataset["scenes"]):
            new_samples = get_find_category_samples(args, scene, category, concepts)
            samples.extend(new_samples)

        samples = get_max_sample(args, samples)
        data = {"samples": samples, "category": category}
        write(args, data, module_name=f"find_{category}")
