#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import combine_masks_for_find
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_find_concept_samples_clevr(args, scene, category, concept):
    objects = [obj for obj in scene["objects"] if "mask" in obj]
    input_mask, target_mask = combine_masks_for_find(args, objects, category, concept)
    sample = {
        "img_name": scene["image_filename"],
        "mask": input_mask,
        "target": target_mask,
    }
    return sample


def get_find_concept_samples_mnist(args, scene, category, concept):
    digits = scene["digits"]
    input_mask, target_mask = combine_masks_for_find(args, digits, category, concept)
    sample = {
        "img_name": scene["img_name"],
        "mask": input_mask,  # is never used
        "target": target_mask,
    }
    return sample


def get_find_concept_samples(args, scene, category, concept):
    if args.experiment == Experiment.CLEVR:
        return get_find_concept_samples_clevr(args, scene, category, concept)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_find_concept_samples_mnist(args, scene, category, concept)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_find_concept_dataset(args):
    categories = args.config.categories
    for category in categories.keys():
        categories = categories[category]
        for concept in categories:
            samples = []
            for scene in tqdm(args.dataset["scenes"]):
                sample = get_find_concept_samples(args, scene, category, concept)
                samples.append(sample)

            samples = get_max_sample(args, samples)
            data = {"samples": samples, "category": category, "concept": concept}
            write(args, data, module_name=f"find_{category}_{concept}")
