#!/usr/bin/env python

import os

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import (
    create_empty_mask,
    create_immediate_relate_mask,
)
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_immediate_relate_category_samples_clevr(args, scene):
    relations = args.config.relate
    samples = []
    objects = scene["objects"]
    relationships = scene["relationships"]
    for relationship in relations:
        for idx, obj in enumerate(objects, 0):
            mask = create_immediate_relate_mask(
                args, idx, objects, relationships, relationship
            )
            sample = {
                "img_name": scene["image_filename"],
                "mask": obj["mask"],
                "relationship": relationship,
                "target": mask,
            }
            samples.append(sample)
    return samples


def get_immediate_relate_category_samples_mnist(args, scene):
    mnist_neighbors = args.config.neighbors
    mnist_relations = args.config.relate
    samples = []
    digits = scene["digits"]
    for relationship in mnist_relations:
        for idx, digit in enumerate(digits, 0):
            neighbor_idx = mnist_neighbors[str(idx)][relationship]
            if neighbor_idx is not None:
                neighbor = digits[int(neighbor_idx)]
                mask = neighbor["mask"]
            else:
                mask = create_empty_mask(args)
            sample = {
                "img_name": scene["img_name"],
                "mask": digit["mask"],
                "relationship": relationship,
                "target": mask,
            }
            samples.append(sample)
    return samples


def get_immediate_relate_category_samples(args, scene):
    if args.experiment == Experiment.CLEVR:
        return get_immediate_relate_category_samples_clevr(args, scene)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_immediate_relate_category_samples_mnist(args, scene)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_immediate_relate_category_dataset(args):
    output_dir = os.path.join(args.output_dir, "immediate_relate")
    os.makedirs(output_dir, exist_ok=True)

    samples = []
    for scene in tqdm(args.dataset["scenes"]):
        new_samples = get_immediate_relate_category_samples(args, scene)
        samples.extend(new_samples)

    samples = get_max_sample(args, samples)
    data = {"samples": samples}
    write(args, data, module_name="immediate_relate")
