#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import create_empty_mask, create_immediate_relate_mask
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_immediate_relate_concept_samples_clevr(args, scene, relationship):
    samples = []
    objects = scene["objects"]
    relationships = scene["relationships"]
    for idx, obj in enumerate(objects):
        # calculate immediate_relations based on relationship_list instead of pixel_coordinates
        target_mask = create_immediate_relate_mask(
            args, idx, objects, relationships, relationship
        )
        sample = {
            "img_name": scene["image_filename"],
            "mask": obj["mask"],
            "target": target_mask,
        }
        samples.append(sample)
    return samples


def get_immediate_relate_concept_samples_mnist(args, scene, relationship):
    mnist_neighbors = args.config.neighbors
    samples = []
    digits = scene["digits"]
    for idx, digit in enumerate(digits, 0):
        neighbor_idx = mnist_neighbors[str(idx)][relationship]
        mask = None
        if neighbor_idx is not None:
            neighbor = digits[int(neighbor_idx)]
            mask = neighbor["mask"]
        else:
            mask = create_empty_mask(args)
        sample = {"img_name": scene["img_name"], "mask": digit["mask"], "target": mask}
        samples.append(sample)
    return samples


def get_immediate_relate_concept_samples(args, scene, relationship):
    if args.experiment == Experiment.CLEVR:
        return get_immediate_relate_concept_samples_clevr(args, scene, relationship)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_immediate_relate_concept_samples_mnist(args, scene, relationship)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_immediate_relate_concept_dataset(args):
    relationships = args.config.relate
    for relationship in relationships:
        samples = []
        for scene in tqdm(args.dataset["scenes"]):
            new_samples = get_immediate_relate_concept_samples(
                args, scene, relationship
            )
            samples.extend(new_samples)
        samples = get_max_sample(args, samples)
        data = {"samples": samples}
        write(
            args,
            data,
            module_name=f"immediate_relate_{relationship}",
        )
