#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_query_category_samples_clevr(scene, category):
    samples = []
    objects = scene["objects"]
    for obj in objects:
        sample = {
            "img_name": scene["image_filename"],
            "mask": obj["mask"],
            "target": obj[category],
        }
        samples.append(sample)
    return samples


def get_query_category_samples_mnist(scene, category):
    samples = []
    digits = scene["digits"]
    for digit in digits:
        sample = {
            "img_name": scene["img_name"],
            "mask": digit["mask"],
            "target": digit[category],
        }
        samples.append(sample)
    return samples


def get_query_category_samples(args, scene, category):
    if args.experiment == Experiment.CLEVR:
        return get_query_category_samples_clevr(scene, category)
    elif args.experiment == Experiment.MNIST_DIALOG:
        return get_query_category_samples_mnist(scene, category)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_query_category_dataset(args):
    categories = args.config.categories
    for category in categories.keys():
        samples = []
        for scene in tqdm(args.dataset["scenes"]):
            new_samples = get_query_category_samples(args, scene, category)
            samples.extend(new_samples)

        samples = get_max_sample(args, samples)
        data = {"samples": samples, "category": category}
        write(args, data, module_name=f"query_{category}")
