#!/usr/bin/env python

import os

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import combine_masks_for_relate
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_relate_category_samples_clevr(args, scene):
    relations = args.config.relate
    objects = scene["objects"]
    relationships = scene["relationships"]
    samples = []
    for relationship in relations:
        for idx, obj in enumerate(objects):
            rel = relationships[relationship][idx]
            input_mask, target_mask = combine_masks_for_relate(args, objects, obj, rel)
            sample = {
                "img_name": scene["image_filename"],
                "mask": input_mask,
                "relationship": relationship,
                "target": target_mask,
            }
            samples.append(sample)
    return samples


def get_relate_category_samples(args, scene):
    if args.experiment == Experiment.CLEVR:
        return get_relate_category_samples_clevr(args, scene)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_relate_category_dataset(args):
    output_dir = os.path.join(args.output_dir, "relate")
    os.makedirs(output_dir, exist_ok=True)

    samples = []
    for scene in tqdm(args.dataset["scenes"]):
        new_samples = get_relate_category_samples(args, scene)
        samples.extend(new_samples)

    samples = get_max_sample(args, samples)
    data = {"samples": samples}
    write(args, data, module_name="relate")
