#!/usr/bin/env python

from tqdm import tqdm

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.generate.utils.masks import combine_masks_for_relate
from nmn.datasets.generate.utils.write import get_max_sample, write


def get_relate_concept_samples_clevr(args, scene, relationship):
    samples = []
    objects = [obj for obj in scene["objects"] if "mask" in obj]
    relationships = scene["relationships"]
    for idx, obj in enumerate(objects):
        rel = relationships[relationship][idx]
        input_mask, target_mask = combine_masks_for_relate(args, objects, obj, rel)
        sample = {
            "img_name": scene["image_filename"],
            "mask": input_mask,
            "target": target_mask,
        }
        samples.append(sample)

    return samples


def get_relate_concept_samples(args, scene, relationship):
    if args.experiment == Experiment.CLEVR:
        return get_relate_concept_samples_clevr(args, scene, relationship)
    else:
        raise UnsupportedExperimentError(args.experiment)


def generate_relate_concept_dataset(args):
    relationships = args.config.relate
    for relationship in relationships:
        samples = []
        for scene in tqdm(args.dataset["scenes"]):
            new_samples = get_relate_concept_samples(args, scene, relationship)
            samples.extend(new_samples)

        samples = get_max_sample(args, samples)
        data = {"samples": samples, "relationship": relationship}
        write(args, data, module_name=f"relate_{relationship}")
