#!/usr/bin/env python

import os

import torch
from skimage import io
from torch.utils.data import Dataset

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.datasets.load.utils.transforms import ImageToTensor
from nmn.utils.config import load_json


def get_dataset(experiment, questions, images_dir, device):
    # only CLEVR is executed with the ProgramEvaluator
    if experiment == Experiment.CLEVR:
        dataset = CLEVRDataset(questions, images_dir, device)
    else:
        raise UnsupportedExperimentError(experiment)
    return dataset


class CLEVRDataset(Dataset):
    def __init__(self, questions, images_dir, device):
        super(CLEVRDataset, self).__init__()
        self.images_dir = images_dir
        self.device = device
        data = load_json(questions)
        self.questions = data["questions"]
        self.image2tensor = ImageToTensor(device)

    def __getitem__(self, idx):
        entry = self.questions[idx]
        # get the image
        img_filename = entry["image_filename"]
        img_path = os.path.join(self.images_dir, img_filename)
        image = io.imread(img_path)
        image = self.image2tensor(image)
        # return the image, program and answer
        if "answer" in entry:
            return image, entry["program"], entry["answer"]
        else:
            return image, entry["program"], None

    def __len__(self):
        return len(self.questions)


def collate_fn(entries):
    image, program, entry = entries[0]
    return torch.stack([image]), program, entry
