#!/usr/bin/env python

import os

from skimage import io

from nmn.datasets.load.module_dataset import ModuleDataset
from nmn.datasets.load.utils.transforms import ImageToTensor, MaskToTensor


class ContextModuleDataset(ModuleDataset):
    def __init__(self, **kwargs):
        super(ContextModuleDataset, self).__init__(**kwargs)
        self.image2tensor = ImageToTensor(kwargs.get("device"))
        self.mask2tensor = MaskToTensor(kwargs.get("device"))

    def __getitem__(self, idx):
        sample = self.samples[idx]
        image_name = sample["img_name"]
        target = sample["target"]

        # load image
        img_path = os.path.join(self.images_dir, image_name)
        image = io.imread(img_path)

        # tensor transformations
        image = self.image2tensor(image)
        target = self.mask2tensor(target)

        source = {"image_name": image_name, "image": image}

        return source, target
