#!/usr/bin/env python

import os

from skimage import io

from nmn.datasets.load.module_dataset import ModuleDataset
from nmn.datasets.load.utils.transforms import ImageToTensor, MaskToTensor


class CountModuleDataset(ModuleDataset):
    def __init__(self, **kwargs):
        super(CountModuleDataset, self).__init__(**kwargs)
        self.image2tensor = ImageToTensor(kwargs.get("device"))
        self.mask2tensor = MaskToTensor(kwargs.get("device"))

        if "mask_noise" in kwargs.get("module_info"):
            self.mask_noise = kwargs.get("module_info").mask_noise
        else:
            self.mask_noise = 0.0

    def __getitem__(self, idx):
        sample = self.samples[idx]
        image_name = sample["img_name"]
        mask = sample["mask"]
        target = sample["target"]

        # load image
        img_path = os.path.join(self.images_dir, image_name)
        image = io.imread(img_path)

        # tensor transformations
        image = self.image2tensor(image)
        mask = self.mask2tensor(mask, noise=self.mask_noise)

        source = {"image_name": image_name, "image": image, "mask": mask}

        return source, target
