#!/usr/bin/env python

import os

from skimage import io

from nmn.datasets.load.module_dataset import ModuleDataset
from nmn.datasets.load.utils.transforms import (
    GloveToTensor,
    ImageToTensor,
    MaskToTensor,
)


class ImmediateRelateCategoryModuleDataset(ModuleDataset):
    def __init__(self, *args, device="cpu", **kwargs):
        super(ImmediateRelateCategoryModuleDataset, self).__init__(
            *args, device=device, **kwargs
        )
        self.image2tensor = ImageToTensor(device)
        self.glove2tensor = GloveToTensor(self.config, device)
        self.mask2tensor = MaskToTensor(device)

    def __getitem__(self, idx):
        sample = self.samples[idx]
        image_name = sample["img_name"]
        mask = sample["mask"]
        relationship = sample["relationship"]
        target = sample["target"]

        # load image
        img_path = os.path.join(self.images_dir, image_name)
        image = io.imread(img_path)

        # to tensors
        image = self.image2tensor(image)
        mask = self.mask2tensor(mask)
        relationship = self.glove2tensor(relationship)
        target = self.mask2tensor(target)

        source = {
            "image_name": image_name,
            "image": image,
            "relationship": relationship,
            "mask": mask,
        }

        return source, target
