#!/usr/bin/env python

import os

from skimage import io

from nmn.datasets.load.module_dataset import ModuleDataset
from nmn.datasets.load.utils.transforms import (
    GloveToTensor,
    ImageToTensor,
    MaskToTensor,
)


class QueryCategoryModuleDataset(ModuleDataset):
    def __init__(self, *args, device="cpu", **kwargs):
        super(QueryCategoryModuleDataset, self).__init__(*args, device=device, **kwargs)
        self.image2tensor = ImageToTensor(device)
        self.glove2tensor = GloveToTensor(self.config, device)
        self.mask2tensor = MaskToTensor(device)
        self.categories_idx = {}
        categories = self.config.categories
        for category in categories:
            self.categories_idx[category] = {}
            for i, concept in enumerate(categories[category]):
                self.categories_idx[category][concept] = i

    def __getitem__(self, idx):
        sample = self.samples[idx]
        image_name = sample["img_name"]
        mask = sample["mask"]
        target = sample["target"]

        # load image
        img_path = os.path.join(self.images_dir, image_name)
        image = io.imread(img_path)

        # to tensors
        image = self.image2tensor(image)
        mask = self.mask2tensor(mask)
        target = self.categories_idx[self.category][target]

        source = {"image_name": image_name, "image": image, "mask": mask}

        return source, target
