#!/usr/bin/env python

import logging
from collections import OrderedDict

import numpy as np


def get_weights(dataloader):
    # count how often each target occurs
    counts = {}
    for sample in dataloader:
        _, target_batch = sample
        for target in target_batch:
            if target in counts:
                counts[target] += 1
            else:
                counts[target] = 1
    # sort the target counts
    ordered_counts = OrderedDict(sorted(counts.items()))
    # transform to arrays and compute the weights
    class_counts = np.array(ordered_counts.values())
    class_weights = np.zeros(len(ordered_counts))
    biggest = np.amax(class_counts)

    for index, count in enumerate(class_counts):
        weight = biggest / count
        class_weights[index] = weight

    for index, count in enumerate(class_counts):
        logging.info(f"Class {index} has {count} samples")

    for index, weight in enumerate(class_weights):
        logging.info(f"Class {index} has weight {weight:.3f} ")
    return class_weights.tolist()


def get_class_weights(ds):
    targets = [sample["target"] for sample in ds.samples]
    unique_targets, counts = np.unique(targets, return_counts=True)
    class_weights = dict(zip(unique_targets, [sum(counts) / c for c in counts]))
    return class_weights
