#!/usr/bin/env python

import cv2
import numpy as np
import torch.nn as nn

from nmn.utils.detectron import load_predictor


class Count(nn.Module):
    def __init__(self, device, **module_kwargs):
        super(Count, self).__init__()
        self.predictor = load_predictor(device, **module_kwargs)

    def forward(self, src):
        image = src["image"]
        mask = src["mask"]

        im = cv2.imread(image)
        im = cv2.cvtColor(im, cv2.COLOR_RGBA2RGB)

        outputs = self.predictor(im)

        mask_list = []
        for predicted_mask in outputs["instances"].pred_masks:
            multiplied_mask = mask.detach().numpy() * predicted_mask.numpy()
            multiplied_mask[multiplied_mask < 0.80] = 0
            if 500 < np.sum(multiplied_mask):
                mask_list.append(multiplied_mask)

        return len(mask_list)
