#!/usr/bin/env python

import torch.nn as nn

from nmn.modules.subsymbolic.mask_operations import MaskIntersection


class Filter(nn.Module):
    def __init__(self, find_modules, embedder):
        super(Filter, self).__init__()
        self.find_modules = find_modules
        self.intersect = MaskIntersection()
        self.embedder = embedder

    def forward(self, src):
        category = src["category"]
        concept = src["concept"]
        # select the correct find module to use
        if category in self.find_modules:
            # category specificity
            find = self.find_modules[category]
        elif concept in self.find_modules:
            # concept specificity
            find = self.find_modules[concept]
        # make the embedding (used in category specificity only)
        x_txt = self.embedder(concept)
        # Run the find module
        attn = find({"image": src["image"], "concept": x_txt.float()})  # TODO fix
        # Do the intersection with the previous mask
        attn = self.intersect(src["mask"], attn)
        return attn
