#!/usr/bin/env python

import numpy as np
import torch
import torch.nn as nn


class Query(nn.Module):
    def __init__(self, query_modules):
        super(Query, self).__init__()
        self.query_modules = query_modules

    def forward(self, src):
        category = src["category"]
        options = src["options"]
        if category in self.query_modules:
            # category specificity
            query = self.query_modules[category]
            log_probs = query({"image": src["image"], "mask": src["mask"]})
            p, index = torch.max(log_probs, dim=1)
            return index.item(), np.exp(p.item())
        else:
            # concept specificity
            # Run all query modules and take the
            # one with the highest prediction for 'yes'
            module_log_probs = torch.full((len(options),), -1.0)
            for i, option in enumerate(options):
                query = self.query_modules[option]
                log_probs = query({"image": src["image"], "mask": src["mask"]})
                # logging.info('Query_%s has probability %f for yes and %f for no'
                #      % (option, np.exp(log_probs[0, 1].item()),
                #         np.exp(log_probs[0, 0].item())))
                # yes has index 1
                yes_prob = log_probs[0, 1].item()
                module_log_probs[i] = yes_prob
            p, index = torch.max(module_log_probs, dim=0)
            return index.item(), np.exp(p.item())
