#!/usr/bin/env python

import torch.nn as nn
import torch.nn.functional as F


class Same(nn.Module):
    def __init__(self, find_modules, query_module, embedder, config):
        super(Same, self).__init__()
        self.query = query_module
        self.find_modules = find_modules
        self.embedder = embedder
        self.config = config

    def forward(self, src):
        category = src["category"]
        index, prob = self.query(src)
        concept = self.config.categories[category][index]
        x_txt = self.embedder(concept)
        if category in self.find_modules:
            # category specificity
            find = self.find_modules[category]
        elif concept in self.find_modules:
            # concept specificity
            find = self.find_modules[concept]
        attn = find({"image": src["image"], "concept": x_txt.float()})
        attn = F.relu(attn - src["mask"])
        return attn
