#!/usr/bin/env python

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.modules.executors.count_detectron import Count as CountDetectron
from nmn.modules.subsymbolic.classifier.classifier import Classifier
from nmn.modules.subsymbolic.segmentation.clevr_context import Context as CLEVRContext
from nmn.modules.subsymbolic.segmentation.mnist_context import Context as MNISTContext
from nmn.modules.subsymbolic.segmentation.semantic_segmentation import (
    SemanticSegmentation,
)
from nmn.modules.utils.module_type import ModuleType, UnsupportedModuleTypeError


def get_module(experiment, device, **module_kwargs):
    """
    Use the module, the specificity, the architecture and the experiment
    to initialise the correct module!
    """
    module_type = module_kwargs.get("type")
    if (
        module_type == ModuleType.FIND
        or module_type == ModuleType.RELATE
        or module_type == ModuleType.IMMEDIATE_RELATE
        or module_type == ModuleType.EXTREME_RELATE
    ):
        net = SemanticSegmentation(**module_kwargs)
    elif module_type == ModuleType.QUERY:
        net = Classifier(**module_kwargs)
    elif module_type == ModuleType.COUNT:
        if experiment == Experiment.CLEVR_DIALOG:
            net = CountDetectron(device, **module_kwargs)  # TODO fix
        elif experiment == Experiment.MNIST_DIALOG or experiment == Experiment.CLEVR:
            net = Classifier(**module_kwargs)
        else:
            raise UnsupportedExperimentError(experiment)
    elif module_type == ModuleType.CONTEXT:
        if experiment == Experiment.CLEVR or experiment == Experiment.CLEVR_DIALOG:
            # TODO fix merge with semantic segmentation
            # issue: uses Down instead of SingleDown
            # also clevr has 'category' specificity! important for unet textencoder loading
            net = CLEVRContext()
        elif experiment == Experiment.MNIST_DIALOG:
            net = MNISTContext()
        else:
            raise UnsupportedExperimentError(experiment)
    else:
        raise UnsupportedModuleTypeError(module_type)
    return net.to(device)
