#!/usr/bin/env python

import logging
import os
from glob import glob

import torch

from nmn.datasets.experiment import Experiment
from nmn.modules.load_modules.get_clevr_modules import get_clevr_evaluation_modules
from nmn.utils.dotdict import DotDict
from scripts.utils.loading import get_model_dir


def get_evaluation_modules(experiment, specificity, config, device):
    """
    This function is used to load the pretrained models for the IRL server.
    It retrieves the names of the modules from the configuration file,
    loads them into memory and then dispatches to specialised functions
    for constructing the neural networks.
    """
    # Determine which pretrained modules need to be loaded
    modules_to_load = config.modules[specificity]
    model_dir = get_model_dir(experiment)
    # Load the pretrained module checkpoints
    checkpoints = []
    for pt_file in glob(os.path.join(model_dir, "*.pt")):
        pt = torch.load(pt_file, map_location=lambda storage, loc: storage)
        if pt["opt"]["module_info"]["name"] in modules_to_load:
            checkpoints.append(pt)
        pt["opt"] = DotDict(pt["opt"])
    logging.info(f"Found {len(checkpoints)} checkpoint files")
    # dispatch to a separate function depending on the experiment
    if experiment == Experiment.CLEVR:
        return get_clevr_evaluation_modules(checkpoints, config, device)
