#!/usr/bin/env python

import torch
import torch.nn as nn
import torch.nn.functional as F


class AlexNet(nn.Module):
    def __init__(self, n_classes):
        super(AlexNet, self).__init__()

        self.features = nn.Sequential(
            # in_channels, out_channels, kernel_size, stride, padding
            nn.Conv2d(3, 64, 3, 2, 1),
            # kernel_size
            nn.MaxPool2d(2),
            nn.ReLU(inplace=True),
            nn.Conv2d(64, 192, 3, padding=1),
            nn.MaxPool2d(2),
            nn.ReLU(inplace=True),
            nn.Conv2d(192, 384, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(384, 256, 3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(256, 256, 3, padding=1),
            nn.MaxPool2d(2),
            nn.ReLU(inplace=True),
        )

        self.classifier = nn.Sequential(
            nn.Dropout(0.5),
            nn.Linear(256 * 20 * 30, 4096),  # TODO fix width and heights
            nn.ReLU(inplace=True),
            nn.Dropout(0.5),
            nn.Linear(4096, 2048),
            nn.ReLU(inplace=True),
            nn.Linear(2048, n_classes),
        )

    def forward(self, src):
        x = torch.mul(src["image"], src["mask"])
        x = self.features(x)
        h = x.view(x.shape[0], -1)
        logits = self.classifier(h)
        log_probs = F.log_softmax(logits, dim=1)
        return log_probs
