#!/usr/bin/env python

import torch.nn as nn
import torch.nn.functional as F

from nmn.modules.subsymbolic.classifier.alex_net import AlexNet
from nmn.modules.subsymbolic.classifier.dense_net import DenseNet
from nmn.modules.subsymbolic.classifier.jerome_net import JeromeNet
from nmn.modules.subsymbolic.classifier.le_net import LeNet
from nmn.modules.subsymbolic.classifier.res_net import ResNet
from nmn.modules.subsymbolic.classifier.squeeze_net import SqueezeNet


def get_classifier_architecture(**kwargs):
    architecture = kwargs.get("architecture")
    if architecture == "JeromeNet":
        return JeromeNet(**kwargs)
    elif architecture == "SqueezeNet":
        return SqueezeNet(**kwargs)
    elif architecture == "AlexNet":
        return AlexNet(**kwargs)
    elif architecture == "DenseNet":
        return DenseNet(**kwargs)
    elif architecture == "ResNet":
        return ResNet(**kwargs)
    elif architecture == "LeNet":
        return LeNet(**kwargs)
    else:
        raise ValueError(f"Unexpected architecture: {architecture}")


class Classifier(nn.Module):
    def __init__(self, **kwargs):
        super(Classifier, self).__init__()
        self.classifier = get_classifier_architecture(**kwargs)

    def forward(self, src):
        logits = self.classifier(src)
        log_probs = F.log_softmax(logits, dim=1)
        return log_probs
