#!/usr/bin/env python

import torch
import torch.nn as nn
from torchvision import models

from nmn.modules.subsymbolic.cnn import ReverseDown


class DenseNet(nn.Module):
    """
    Count module based on torchvision's DenseNet model.
    """

    def __init__(self, **kwargs):
        super(DenseNet, self).__init__()

        n_classes = kwargs.get("n_classes")
        self.img_conv = ReverseDown(3, 3)  # 3 image channels
        self.att_conv = ReverseDown(1, 3)  # 1 attention channel

        self.densenet = models.densenet121(pretrained=False)
        self.densenet.classifier = nn.Sequential(
            nn.Linear(self.densenet.classifier.in_features, 128),
            nn.ReLU(),
            nn.Linear(128, 64),
            nn.ReLU(),
            nn.Linear(64, n_classes),
        )

    def forward(self, src):
        img = self.img_conv(src["image"])
        att = self.att_conv(src["mask"])
        x = torch.mul(img, att)
        return self.densenet(x)
