#!/usr/bin/env python

import torch
import torch.nn as nn
import torch.nn.functional as F


class LeNet(nn.Module):
    def __init__(self, **kwargs):
        super(LeNet, self).__init__()

        n_classes = kwargs.get("n_classes")  # depends on the problem
        img_width = kwargs.get("img_width")  # width of the original image
        img_height = kwargs.get("img_height")  # height of the original image

        self.conv1 = nn.Conv2d(in_channels=3, out_channels=6, kernel_size=5)
        self.conv2 = nn.Conv2d(in_channels=6, out_channels=16, kernel_size=5)
        self.conv3 = nn.Conv2d(in_channels=16, out_channels=32, kernel_size=5)
        self.conv4 = nn.Conv2d(in_channels=32, out_channels=64, kernel_size=5)
        self.pool = nn.MaxPool2d(2, 2)
        # (2 ** 4) -> architecture uses 4 MaxPool operations
        self.fc_1 = nn.Linear(
            64 * (img_width // (2 ** 4)) * (img_height // (2 ** 4)), 128
        )
        self.fc_2 = nn.Linear(128, 64)
        self.fc_3 = nn.Linear(64, n_classes)

    def forward(self, src):
        x = torch.mul(src["image"], src["mask"])
        x = F.relu(self.pool(self.conv1(x)))
        x = F.relu(self.pool(self.conv2(x)))
        x = F.relu(self.pool(self.conv3(x)))
        x = F.relu(self.pool(self.conv4(x)))
        x = x.view(x.shape[0], -1)
        x = F.relu(self.fc_1(x))
        x = F.relu(self.fc_2(x))
        logits = self.fc_3(x)
        log_probs = F.log_softmax(logits, dim=1)
        return log_probs
