#!/usr/bin/env python

import torch
import torch.nn as nn
from torchvision import models

from nmn.modules.subsymbolic.cnn import ReverseDown


class ResNet(nn.Module):
    """
    Count module based on torchvision's ResNet model.
    uses ResNet34.
    """

    def __init__(self, **kwargs):
        super(ResNet, self).__init__()

        n_classes = kwargs.get("n_classes")
        self.img_conv = ReverseDown(3, 3)  # 3 image channels
        self.att_conv = ReverseDown(1, 3)  # 1 attention channel

        self.resnet = models.resnet34(pretrained=False)
        self.resnet.fc = nn.Linear(self.resnet.fc.in_features, n_classes)

    def forward(self, src):
        img = self.img_conv(src["image"])
        att = self.att_conv(src["mask"])
        x = torch.mul(img, att)
        return self.resnet(x)
