#!/usr/bin/env python

import torch
import torch.nn as nn
from torchvision import models

from nmn.modules.subsymbolic.cnn import ReverseDown
from nmn.modules.utils.module_type import ModuleType


class SqueezeNet(nn.Module):
    """
    Count module based on torchvision's SqueezeNet model.
    """

    def __init__(self, **kwargs):
        super(SqueezeNet, self).__init__()
        n_classes = kwargs.get("n_classes")
        self.squeezenet = models.squeezenet1_1(pretrained=False)
        self.squeezenet.classifier._modules["1"] = nn.Conv2d(
            512, n_classes, kernel_size=(1, 1)
        )
        self.squeezenet.num_classes = n_classes
        # TODO query and count do not match up in CLEVR
        # ensures backwards compatibility
        # for example add that new query also use img_conv/att_conv's layers
        # or (
        #    self.module_type == ModuleType.QUERY and self.new_squeeze
        # ):
        self.module_type = kwargs.get("type")
        if self.module_type == ModuleType.COUNT:
            self.img_conv = ReverseDown(3, 3)
            self.att_conv = ReverseDown(1, 3)

    def forward(self, src):

        if self.module_type == ModuleType.COUNT:
            img = self.img_conv(src["image"])
            att = self.att_conv(src["mask"])
        else:
            img = src["image"]
            att = src["mask"]
        x = torch.mul(img, att)
        return self.squeezenet(x)
