#!/usr/bin/env python

import torch
import torch.nn as nn

from nmn.modules.subsymbolic.cnn import DoubleConv, Down, OutConv, Up


class Context(nn.Module):
    def __init__(self):
        super(Context, self).__init__()

        channel_complexity = 4
        self.inc = DoubleConv(3, channel_complexity)  # 3 image channels
        self.down = Down(channel_complexity, channel_complexity)
        self.up = Up(channel_complexity * 2, channel_complexity)
        self.outc = OutConv(channel_complexity, 1)  # 1 attention channel

    def forward(self, src):
        x = src["image"]
        x1 = self.inc(x)
        x2 = self.down(x1)
        x = self.up(x2, x1)
        logits = self.outc(x)
        attn = torch.sigmoid(logits)
        return attn
