#!/usr/bin/env python


from nmn.options.options import Options, none_or_value


class EvalOptions(Options):
    """
    EvalOptions are used to evaluate a complete network of modules.
    This is only relevant for CLEVR! Both CLEVR_Dialog and
    MNIST_Dialog require interaction with IRL for evaluation.
    """

    def __init__(self):
        super(EvalOptions, self).__init__()

        self.parser.add_argument("--max_samples", type=none_or_value, default=None)
        self.parser.add_argument(
            "--specificity",
            type=str,
            required=True,
            choices=["category", "concept"],
            default="category",
            help="Which specificity to evaluate",
        )
        self.parser.add_argument(
            "--export_trace_pkl",
            type=none_or_value,
            default=None,
            help="Path to export all outputs to",
        )
        self.parser.add_argument(
            "--export_answers",
            type=none_or_value,
            default=None,
            help="Path to export the answers",
        )
        self.parser.add_argument(
            "--use_clean_count_masks",
            action="store_true",
            help="Discretize the count masks or not",
        )
        self.parser.add_argument(
            "--use_clean_query_masks",
            action="store_true",
            help="Discretize the query masks or not",
        )
        self.parser.add_argument(
            "--count_clean_mask_threshold", type=float, default=0.9
        )
        self.parser.add_argument(
            "--query_clean_mask_threshold", type=float, default=0.5
        )
        self.parser.add_argument(
            "--evaluation_mode",
            type=str,
            choices=["individual", "grouped"],
            default="individual",
            help="Method by which to measure the performance of modules.",
        )
