#!/usr/bin/env python

import logging

import torch

from nmn.options.eval_options import EvalOptions
from nmn.options.server_options import ServerOptions
from nmn.options.test_options import TestOptions
from nmn.options.train_options import TrainOptions


def get_options(mode):
    if mode == "train":
        opt = TrainOptions().parse()
    elif mode == "test":
        opt = TestOptions().parse()
    elif mode == "server":
        opt = ServerOptions().parse()
    elif mode == "eval":
        opt = EvalOptions().parse()

    # Add correct device to opt
    opt.device = None
    if opt.use_cuda and torch.cuda.is_available():
        logging.info("CUDA device available")
        opt.device = "cuda:0"
    else:
        logging.info("No CUDA device available")
        opt.device = "cpu"

    return opt
