#!/usr/bin/env python

import argparse

from nmn.datasets.experiment import Experiment


class Options:
    """
    The Options class specifies the command line options
    that are applicable for training and testing modules
    and for evaluating the modules in complete programs.
    """

    def __init__(self):
        self.parser = argparse.ArgumentParser()
        # experiment
        self.parser.add_argument(
            "--experiment",
            required=True,
            type=str,
            choices=Experiment.list(),
            help="Name of the experiment",
        )
        # technical stuff
        self.parser.add_argument(
            "--use_cuda",
            action="store_true",
            help="Whether to use cuda or not",
        )
        self.parser.add_argument(
            "--num_workers",
            default=0,
            type=int,
            help="number of workers for loading data",
        )
        self.parser.add_argument(
            "--shuffle_data",
            action="store_true",
            help="shuffle dataloader",
        )

    def parse(self):
        opt = self.parser.parse_args()
        return opt


def none_or_value(value):
    if value == "None":
        return None
    elif value.isdigit():
        return int(value)
    return value
