#!/usr/bin/env python


from nmn.options.options import Options, none_or_value


class TrainOptions(Options):
    """
    TrainOptions are the command line arguments for training modules
    """

    def __init__(self):
        super(TrainOptions, self).__init__()

        # category options
        self.parser.add_argument(
            "--num_epochs", default=100, type=int, help="amount of epochs"
        )
        self.parser.add_argument(
            "--max_samples_train",
            type=none_or_value,
            default=None,
            help="amount of samples used during training",
        )
        self.parser.add_argument(
            "--max_samples_val",
            type=none_or_value,
            default=None,
            help="amount of samples used during val",
        )

        # we add 2 subparsers: one for training a model from scratch
        # and another for resuming training for a model
        self.subparsers = self.parser.add_subparsers(dest="mode")

        # ===> train <===
        self.train_subparser = self.subparsers.add_parser("train")

        self.train_subparser.add_argument("--batch_size", default=128, type=int)
        # module to train
        self.train_subparser.add_argument(
            "--module_config",
            required=True,
            type=str,
            help="Config of the module to train.",
        )
        self.train_subparser.add_argument(
            "--module_name",
            required=True,
            type=str,
            help="Long name of the to be trained module.",
        )
        # training parameters
        self.train_subparser.add_argument(
            "--learning_rate", default=1e-3, type=float, help="learning rate"
        )
        self.train_subparser.add_argument(
            "--weight_decay", default=0.0, type=float, help="weight decay"
        )
        self.train_subparser.add_argument("--balance_classes", action="store_true")
        self.train_subparser.add_argument("--mask_noise", default=0.0, type=float)

        # early stopping params
        self.train_subparser.add_argument("--use_early_stopping", action="store_true")
        self.train_subparser.add_argument(
            "--early_stopping_patience", default=3, type=int
        )
        self.train_subparser.add_argument(
            "--early_stopping_delta", default=0.0, type=float
        )
        self.train_subparser.add_argument(
            "--early_stopping_verbose", action="store_true"
        )

        # console 'logging' info
        self.train_subparser.add_argument(
            "--print_every",
            default=50,
            type=int,
            help="display training info every N samples",
        )
        self.train_subparser.add_argument(
            "--record_every", default=1, type=int, help="record loss every N samples"
        )
        self.train_subparser.add_argument(
            "--export_mode",
            default="best",
            type=str,
            choices=["all", "last", "best"],
            help="which checkpoints are exported",
        )

        # ===> resume <===
        self.resume_subparser = self.subparsers.add_parser("resume")
        self.resume_subparser.add_argument(
            "--checkpoint",
            required=True,
            type=str,
            help="checkpoint to resume training from",
        )
