#!/usr/bin/env python

import logging


class EarlyStopping:
    def __init__(self, patience=7, delta=0, verbose=False):
        self.patience = patience
        self.verbose = verbose
        self.counter = 0
        self.best_loss = None
        self.early_stop = False
        self.delta = delta

    def __call__(self, validation_loss):
        score = -validation_loss
        if self.best_loss is None:
            if self.verbose:
                logging.info(f"Validation loss decreases: {100} -> {score}")
            self.best_loss = score
        elif score < self.best_loss + self.delta:
            self.counter += 1
            if self.verbose:
                logging.info(f"Validation loss did not decrease")
                logging.info(f"Early stopping counter: {self.counter}/{self.patience}")
            if self.counter >= self.patience:
                self.early_stop = True
        else:
            if self.verbose:
                logging.info(
                    f"Validation loss decreases: {-self.best_loss} -> {-score}"
                )
            self.best_loss = score
            self.counter = 0
        return self.early_stop
