import json
import logging

import numpy as np

from nmn.datasets.experiment import Experiment, UnsupportedExperimentError
from nmn.utils.dotdict import DotDict


def load_experiment_config(opt):
    logging.info("Reading experiment config file...")
    if opt.experiment == Experiment.MNIST_DIALOG:
        config_file = "data/mnist_dialog/config.json"
    elif (
        opt.experiment == Experiment.CLEVR or opt.experiment == Experiment.CLEVR_DIALOG
    ):
        config_file = "data/clevr/config.json"
    else:
        raise UnsupportedExperimentError(opt.experiment)

    data = load_json(config_file)
    opt.config = DotDict(data)
    weights = np.array(list(opt.config.embeddings.values()), dtype=float)
    opt.config.weights = weights
    logging.info("Experiment config loaded.")


def load_module_config(opt):
    logging.info("Reading module config file...")
    cfg_path = f"data/{opt.experiment}/architectures/{opt.module_config}"
    data = load_json(cfg_path)
    opt.module_info = DotDict(data)
    logging.info(f"Module config {cfg_path} loaded.")


def load_json(path):
    with open(path, "r") as f:
        return json.load(f)
