import datetime
import logging
import os
import shutil
from pprint import pformat

import dateutil
import dateutil.tz


class Logger:
    def __init__(self, opt, mode) -> None:
        now = str(datetime.datetime.now(dateutil.tz.tzlocal())).replace(" ", "_")
        opt.logdir = os.path.join("logging", now)
        os.makedirs(opt.logdir, exist_ok=True)
        logfile = os.path.join(opt.logdir, "logfile.log")

        # create logger
        logger = logging.getLogger()
        logger.setLevel(mode)

        # create filehandler and set level to info
        fh = logging.FileHandler(logfile)
        fh.setLevel(mode)
        # "[%(asctime)s - %(levelname)s]\n > %(message)s", datefmt="%H:%M:%S"
        formatter = logging.Formatter("%(message)s", datefmt="%H:%M:%S")
        fh.setFormatter(formatter)
        logger.addHandler(fh)

        # set stream handler to same format
        root_logger = logging.getLogger()  # no name
        for handler in root_logger.handlers:
            if isinstance(handler, logging.StreamHandler):
                handler.setFormatter(formatter)


def setup_logger(opt):
    """Logs the experiment for reproducibility, returns the

    For each experiment a new unique directory is created.
    A copy of the code, config file and the stdout is copied.

    opt:
        opt (dict): command-line arguments
        cfg (dict): contains the parameters of the experiment to run
        logdir (str): path of the folder where the experiment is logged

    Returns:
        Logger: handles logging administration
    """
    Logger(opt, logging.INFO)
    logging.info(f" === Saving output to: {opt.logdir} === ")
    logging.info(" === Using config === ")
    logging.info(pformat(vars(opt)))  # log raw command-line opt

    # copy nmn codebase
    code_dir_name = "nmn"
    code_dir = os.path.join(os.getcwd(), code_dir_name)
    shutil.copytree(
        code_dir,
        os.path.join(opt.logdir, code_dir_name),
        ignore=shutil.ignore_patterns("*.pyc", "tmp*"),
    )
    shutil.make_archive(
        os.path.join(opt.logdir, "nmn"),
        "zip",
        os.path.join(opt.logdir, code_dir_name),
    )
    shutil.rmtree(os.path.join(opt.logdir, code_dir_name))

    # copy scripts
    code_dir_name = "scripts"
    code_dir = os.path.join(os.getcwd(), code_dir_name)
    shutil.copytree(
        code_dir,
        os.path.join(opt.logdir, code_dir_name),
        ignore=shutil.ignore_patterns("*.pyc", "tmp*"),
    )
    shutil.make_archive(
        os.path.join(opt.logdir, "scripts"),
        "zip",
        os.path.join(opt.logdir, code_dir_name),
    )
    shutil.rmtree(os.path.join(opt.logdir, code_dir_name))
