#!/usr/bin/env python

import argparse
import logging

from nmn.datasets.experiment import Experiment
from nmn.datasets.generate.context import generate_context_dataset
from nmn.datasets.generate.count import generate_count_dataset
from nmn.datasets.generate.extreme_relate_concept import generate_extreme_relate_concept_dataset
from nmn.datasets.generate.find_category import generate_find_category_dataset
from nmn.datasets.generate.find_concept import generate_find_concept_dataset
from nmn.datasets.generate.immediate_relate_category import generate_immediate_relate_category_dataset
from nmn.datasets.generate.immediate_relate_concept import generate_immediate_relate_concept_dataset
from nmn.datasets.generate.query_category import generate_query_category_dataset
from nmn.datasets.generate.query_concept import generate_query_concept_dataset
from nmn.datasets.generate.relate_category import generate_relate_category_dataset
from nmn.datasets.generate.relate_concept import generate_relate_concept_dataset
from nmn.modules.utils.module_type import ModuleType
from nmn.utils.config import load_experiment_config, load_json

parser = argparse.ArgumentParser()
parser.add_argument(
    "--experiment",
    required=True,
    choices=Experiment.list(),
    help="The config file of the experiment.",
)
parser.add_argument(
    "--max_samples", type=int, default=None, help="Maximum number of samples to export"
)


def main(args):
    """
    Generate the masks for the training data
    """
    # load config into args
    load_experiment_config(args)

    for dataset_name in ["train", "val", "test"]:
        # read the dataset
        logging.info(f"Reading dataset {dataset_name}...")
        args.scenes_basename = dataset_name
        args.dataset = load_json(f"data/{args.experiment}/scenes/{dataset_name}.json")
        args.output_dir = f"data/{args.experiment}/module_data/"

        module_types = args.config.trained_modules
        # make the datasets
        if ModuleType.COUNT in module_types:
            logging.info(f"Generating {ModuleType.COUNT} dataset...")
            generate_count_dataset(args)
        if ModuleType.CONTEXT in module_types:
            logging.info("Generating context dataset...")
            generate_context_dataset(args)
        if ModuleType.FIND in module_types:
            logging.info("Generating find category dataset...")
            generate_find_category_dataset(args)
            logging.info("Generating find concept dataset...")
            generate_find_concept_dataset(args)
        if ModuleType.QUERY in module_types:
            logging.info("Generating query category dataset...")
            generate_query_category_dataset(args)
            logging.info("Generating query concept dataset...")
            generate_query_concept_dataset(args)
        if ModuleType.RELATE in module_types:
            logging.info("Generating relate category dataset...")
            generate_relate_category_dataset(args)
            logging.info("Generating relate concept dataset...")
            generate_relate_concept_dataset(args)
        if ModuleType.IMMEDIATE_RELATE in module_types:
            logging.info("Generating immediate relate category dataset...")
            generate_immediate_relate_category_dataset(args)
            logging.info("Generating immediate relate concept dataset...")
            generate_immediate_relate_concept_dataset(args)
        if ModuleType.EXTREME_RELATE in module_types:
            logging.info("Generating extreme relate concept dataset...")
            generate_extreme_relate_concept_dataset(args)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
