#!/usr/bin/env python

# script to generate raw MNIST masks dataset that is served to the dataset generation scripts (generate_data.py)
# overwrites the given input file

import argparse
import json
import logging
import os

from tqdm import tqdm

from nmn.datasets.generate.utils.masks import create_mnist_mask_by_index
from nmn.utils.config import load_json

parser = argparse.ArgumentParser()
parser.add_argument(
    "--scenes",
    default="data/mnist_dialog/scenes/",
    help="Path to the scenes directory.",
)


def main(args):
    """Generate the masks for the split dialog data.

    This process is destructive, i.e., it overwrites the file containing the data.
    """

    datasets = ["train", "val", "test"]
    for dataset in datasets:
        # read the dataset
        logging.info(f"Reading {dataset} dataset...")
        fpath = os.path.join(args.scenes, f"{dataset}.json")
        args.dataset = load_json(fpath)

        # retrieve the scene metadata
        data = {
            "gridSize": args.dataset["gridSize"],
            "seqlen": args.dataset["seqlen"],
            "seed": args.dataset["seed"],
            "QAperImg": args.dataset["QAperImg"],
            "size": args.dataset["size"],
            "scenes": [],
        }

        logging.info("Processing scenes...")

        masks = []
        for idx in range(16):
            masks.append(create_mnist_mask_by_index(idx))

        for idx, scene in enumerate(tqdm(args.dataset["images"]), 0):
            digits = [item for sublist in scene for item in sublist]
            for index, digit in enumerate(digits, 0):
                digit["mask"] = masks[index]
                digit["number"] = str(digit["number"])
            scene = {"digits": digits, "img_name": str(idx).zfill(5) + ".jpg"}
            data["scenes"].append(scene)

        with open(fpath, "w") as f:
            json.dump(data, f)

        logging.info("Mask generation successful!")


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
