#!/usr/bin/env python

import argparse
import json
import logging
import os

from tqdm import tqdm

from nmn.utils.config import load_json

parser = argparse.ArgumentParser()
parser.add_argument(
    "--dataset_json",
    type=str,
    default="data/mnist_dialog/dataset.json",
    help="Contains the original MNIST-dialog scene and dialog information.",
)


def main(args):
    """
    Split the dialog and scene annotations along a train;val;test split.
    """
    dialogs_dir = "data/mnist_dialog/dialogs/"
    scenes_dir = "data/mnist_dialog/scenes/"

    logging.info("Reading dataset...")
    args.dataset = load_json(args.dataset_json)

    # split and save the dialogs according to the image index ('img' field)
    datasets = ["train", "valid", "test"]
    sizes = [30000, 10000, 10000]

    logging.info("Splitting dialogs...")
    for idx, dataset in enumerate(datasets, 0):
        sentences = args.dataset[f"{dataset}Examples"]
        for index in tqdm(range(0, sizes[idx] - 1, 3)):
            triple = {
                "dialog1": sentences[index],
                "dialog2": sentences[index + 1],
                "dialog3": sentences[index + 2],
            }
            # rename valid to val
            if dataset == "valid":
                dataset = "val"
            img_name = str(sentences[index]["img"]).zfill(5)
            out_dir = os.path.join(dialogs_dir, dataset)
            os.makedirs(out_dir, exist_ok=True)
            fpath = os.path.join(out_dir, f"MNIST_{dataset}_{img_name}.json")
            with open(fpath, "w") as f:
                json.dump(triple, f)

    # split image metadata annotations by splits
    logging.info("Split image annotations...")
    for dataset in datasets:
        split = {
            "QAperImg": args.dataset["QAperImg"],
            "gridSize": args.dataset["gridSize"],
            "seqlen": args.dataset["seqlen"],
            "seed": args.dataset["seed"],
            "size": args.dataset[f"{dataset}Size"],
            "images": args.dataset[f"{dataset}Images"],
        }

        os.makedirs(scenes_dir, exist_ok=True)
        # rename valid to val
        if dataset == "valid":
            dataset = "val"
        fpath = os.path.join(scenes_dir, f"{dataset}.json")
        with open(fpath, "w") as f:
            json.dump(split, f)

    logging.info("Split successful!")


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
